//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

PORTIONS OF CODE FROM DUKE NUKEM 3D SOURCE CODE
*/
//-------------------------------------------------------------------------

// game_main.cpp

#include "game.h"

hiddensprites Game::InvisibleSprites[] = {
	// StartTile       EndTile     type
	{2522,             2529,         -1},
	{2331,             2332,         -1},
	{2519,             2521,         -1},
	{-1,                 -1,        416},

	{-1,                 -1,         -1}
};

Game::Game()
{
	initframe      = 0;
	thisWeaponPic  = 0;
	ingame         = FALSE;	
	exit_game      = FALSE;
	scripts_loaded = FALSE;
	loadingplague  = FALSE;
	multiplayer    = FALSE;
	noclip         = FALSE;
	consoleactive  = FALSE;
	NodrawWeapon   = FALSE;
	gameThread     = 0;
	numSectors     = 0;
	numofsprites   = 0;
	expansionLists = 0;
	skippageflip   = 0;
	anisotropy     = 1;
	resolution     = 1;  // 640x480
	fullscreen     = 1;  // fullscreen on
	thisRenderer   = 0;  // 8bit renderer
	brightness     = 30; // default brightness
	autoMap        = FALSE;
	jumpingtics    = 0.0; // player's not jumping atm....
	jumpturn       = 0;
	numofsprites   = 0;
	forcedProjWidth  = 0;
	forcedProjHeight = 0;
	thislogFile = fopen("log.TXT","wb");
}

/*
===============================================
void Game::WriteBloodCFG(gameCFG_t *thisCfg)
===============================================
*/

void Game::WriteBloodCFG(gameCFG_t *thisCfg)
{
	FILE *bloodCFG;
	const int cfgVersion = 3;
	
	bloodCFG = fopen("winblood.cfg","wb");
	fseek(bloodCFG,0,SEEK_SET);
	//fwrite((void *)cfgVersion,sizeof(int),1,bloodCFG);
	fwrite(thisCfg,sizeof(gameCFG_t),1,bloodCFG);
	fclose(bloodCFG);
}

/*
===============================
void Game::ReadBloodCFG(void)
===============================
*/

void Game::ReadBloodCFG(void)
{
	FILE *bloodcfg;
	gameCFG_t *thisCfg = new gameCFG_t;

	bloodcfg = fopen("winblood.cfg","rb");
	if(!bloodcfg)
		return;
	fseek(bloodcfg,0,SEEK_SET);
	fread(thisCfg,sizeof(gameCFG_t),1,bloodcfg);

	this->resolution   = thisCfg->resolution;
	this->anisotropy   = thisCfg->antistrophy;
	this->thisRenderer = thisCfg->renderer;
	this->fullscreen   = thisCfg->fullscreen;

	// Change the resolutiond data from defaults
	menuManager->ChangeResolution(this->resolution,this->anisotropy,this->fullscreen,
								  this->thisRenderer);
	fclose(bloodcfg);
}

/*
====================================
void Game::init_system(void)
====================================
*/
void Game::init_system(void)
{
	printf("Initlizing Game....\n");
	build_func->Load_Pallete("LOOKUP.DAT");

	printf("Prepare to enter Blood....\n");
}
/*
===================================
void Game::Error(char *err, ...)
===================================
*/
void Game::Error(char *err, ...)
{
	char temp[10];
	build_func->SDL_Quit();
	printf(err);
	gets(temp);
	exit(0);
}

/*
=================================
void Game::OperateSpritesAuto(void)
=================================
*/

void Game::OperateSpritesAuto(void)
{
	bloodPlayer_t *thisPlayer   = gamePlayers.at(0);

	if(build_func->playerTouchSprite(thisPlayer->posx,thisPlayer->posy,thisPlayer->posz,
									 thisPlayer->sectorNum,2169))
	{
		thisPlayer->health += 20;
		if(thisPlayer->health > 100)
			thisPlayer->health = 100;
	}
}

/*
=================================
void Game::OperateSprites(void)
=================================
*/

void Game::OperateSprites(void)
{
	short neartagsector        = 0;
	short neartagwall          = 0;
	short neartagsprite        = 0;
	long  neartaghitdist       = 0;
	bloodPlayer_t *thisPlayer   = gamePlayers.at(0);
	int nextLevel;
	spritetype *thisSprite;
	char temptxt[100];
	build_func->neartag(thisPlayer->posx,thisPlayer->posy,thisPlayer->posz,thisPlayer->sectorNum,thisPlayer->ang,&neartagsector,&neartagwall,&neartagsprite,&neartaghitdist,1024L,3);
	
	thisSprite = build_func->Game_GetSprite(neartagsprite);

	switch(thisSprite->picnum)
	{
		case 318: // end of level sprite
			menuManager->pLevel  += 1;
			nextLevel   = menuManager->pLevel   + 1;
			sprintf(temptxt,"e%dm%d.map",menuManager->pEpisode+1,nextLevel);
            LoadBoard(temptxt);
		break;
		case 1078: // lightsiwtch off
			thisSprite->picnum = 1079;
			build_func->build_ClearKey(sc_Space);
		break;
		case 1079: // lightswitch on
			thisSprite->picnum = 1078;
			build_func->build_ClearKey(sc_Space);
		break;
	}
}

/*
================================
void Game::LoadingPlague(void)
================================
*/

void Game::LoadingPlague(void)
{
	loadingplague = TRUE;
	//while(loadingplague)
	//	Sleep(5);
}

/*
===================================
void Game::EndLoadingPlague(void)
===================================
*/

void Game::EndLoadingPlague(void)
{
	loadingplague = FALSE;
}

/*
=============================
void Game::InitExtras(void)
=============================
*/

void Game::InitExtras(void)
{
	spritetype *thisSprite, *newSprite;
	long newspr;
	for(int i = 0; i < numofsprites; i++)
	{
		thisSprite = build_func->Game_GetSprite(i);

		switch(thisSprite->picnum)
		{
			case SPRTOURCH: // spawn fire for the torch
				newspr = build_func->spawnsprite(0, thisSprite->x,thisSprite->y, thisSprite->z-25,2,0,0,32,64,64,0,0,2101,thisSprite->ang,
										0,0,0,thisSprite->sectnum+4096,thisSprite->sectnum,4,63,0,0);					                    
			//	if(newspr >= numofsprites)
			//		numofsprites++;
			break;
		}
	}
}

/*
==========================
void Game::KillAllThreads(void)
==========================
*/

void Game::KillAllThreads(void)
{
	printf(".....Reseting game threads\n");
	// clear monsters
	for(int i = 0; i < worldMobs.size(); i++)
	{
		worldMobs.at(i)->thisMob->mobAI->KillThread();
	}
	registeredAI.clear();

	// Clear weapons
	for(int i = 0; i < playerWeapons.size(); i++)
	{
		playerWeapons.at(i)->KillThread();
	}
	playerWeapons.clear();
}

/*
==================================================================
void Game::SpriteAnimation(int spritenum, int startTile, int frames, int time, int x, int y)
==================================================================
*/

void Game::SpriteAnimation(int spritenum, int startTile, int frames, int time, int x, int y)
{
	//build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
}

/*
====================================
void Game::LoadBoard(char *board)
====================================
*/
void Game::LoadBoard(char *board)
{
	bloodPlayer_t *thisPlayer = new bloodPlayer_t; // recreate player 0
	printf("===============GAME INIT================\n");

	ingame = false;
	soundManager->StopAllsounds();
	soundManager->FlushSystem();
	thisBoard    = board;
	menuManager->SetMenu(BLOODMENU_LOADING);
	menuManager->DrawMenu();
	build_func->nextpage();
//	numofsprites = 0;

	if(gameThread == NULL)
		gameThread = new thread_t(new program_t("game.c"));
	else
		gameThread->ShutdownThread();  // wait for level to load

	//ShutdownAI();
	KillAllThreads();			   // clear all running threads(scripts).
	printf("....clearing game databases\n");
	gamePlayers.clear();           // players get clearned on new level
	worldMobs.clear();			   // flush world mobs
	registeredAI.clear();		   // clear registered AI
	worldEvents.clear();		   // clear all world events
	deadSprites.clear();		   // clear list of dead mobs
	activeProjectiles.clear();     // clear projectile list
	activeSpecials.clear();        // clear specials list
	srand(time(NULL));			   // reset the random number generator
	guimanager->DrawLoading(board);
	ai_Shutdown();
	skippageflip = 0;
	if(scripts_loaded == FALSE)
	{
		printf("Running System Script\n");
		thread_t *gameInit = new thread_t(new program_t("system.c"));
		gameInit->RunProgram();
		gameInit->WaitFor_Thread();
	}
	
	// Create the player on this client
	thisPlayer->health   = 100;

	printf("Loading game board\n");
	build_func->Decrypt_Map(board,"temp.map");
	
	if(build_func->loadboard( "temp.map",0,&thisPlayer->posx, &thisPlayer->posy, &thisPlayer->posz, 
							  &thisPlayer->ang,&thisPlayer->sectorNum ) == -1 )
	{
		Error("LoadBoard: failed to find converted level 'temp.map'\n");
	}

	printf("....Putting player at %d %d %d\n",thisPlayer->posx,thisPlayer->posy,thisPlayer->posz);
	gamePlayers.push_back(thisPlayer); // assumed that it is the current player

	printf(".....setting default player and sector states\n");
	SetPlayerWeapon(0); // pitchfork

	RemoveEffectors();
	SetSpriteStates();  // Make all doors closed, etc
	InitExtras();
	printf(".....precaching sounds\n");
	PrecacheSounds();
	printf(".....starting game threads\n");
	gameThread->RunProgram();
	InternalRegisterAI();
	//StartAI();
//	Sleep(90); // wait for threads to sync with parent process
	soundManager->PlayMusic(levelMusic);
	printf("Entering game.......\n");
	ingame = TRUE;

	menuManager->SetMenu(BLOODMENU_OFF);
	if(multiplayer == TRUE)
		soundManager->soundPlay(0);
}

/*
============================
void Game::LoadVoxels(void)
============================
*/

void Game::LoadVoxels(void)
{

}


/*
=================================
void Game::RunProjectiles(void)
=================================
*/

void Game::RunProjectiles(void)
{
	spritetype   *thisSprite;
	jprojectTile *thisProjectile, *thisSpecial;
	short sect=0, hitsect=0, hitspr=0, hitwall=0;
	long zvel=0,  hitx=0,    hity=0,   hitz=0;
	long sina,cosa,momx,momy;
	long fvel = -320;
	long hitobj = 0;
	long newSpecial = 0;
	long oldz;

	for(int i = 0; i < activeProjectiles.size(); i++)
	{
		thisProjectile = activeProjectiles.at(i);
		thisSprite     = build_func->unsafeGetSprite((short)thisProjectile->spritenum);

		sina = (sin((long double)thisSprite->ang * (3.141592/1024)) * 16383);
		cosa = (cos((long double)thisSprite->ang * (3.141592/1024)) * 16383);

		oldz = thisSprite->z; // prevent bouncy rockets

		build_func->hitscan(thisSprite->x,thisSprite->y,thisSprite->z,thisSprite->sectnum,
                cosa,
                sina,
                zvel<<6,&hitsect,&hitwall,&hitspr,&hitx,&hity,&hitz,CLIPMASK1);


		hitobj = MoveSprite(thisSprite,40,40,CLIPMASK1);

		thisSprite->z = oldz;

		// Hit a wall or sprite
        if(hitobj != 0)
		{
			thisSpecial = new jprojectTile;

			thisSprite->picnum = 4580;
			if(thisProjectile->explosion == 1)
			{
				//build_func->deletesprite((short)thisProjectile->spritenum);
				newSpecial = build_func->spawnsprite(0, thisSprite->x,thisSprite->y, thisSprite->z+300,2,0,0,32,64,64,0,0,984,thisSprite->ang,
											0,0,0,thisSprite->sectnum+4096,thisSprite->sectnum,4,63,0,0);					                    

				//MoveSprite(build_func->Game_GetSprite(newSpecial),1,1,CLIPMASK1); // makesure it is inside the level
				thisSpecial->damage    = thisProjectile->damage * 2;
				thisSpecial->lastpic   = 984;
				thisSpecial->spritenum = newSpecial;
				activeSpecials.push_back(thisSpecial);
				soundManager->soundPlay(19);
			}
			activeProjectiles.erase(activeProjectiles.begin() + i);
			i--; // dont skip a projectile
			continue;
		}
	}
}

/*
==============================
void Game::RunSpecials(void)
==============================
*/

void Game::RunSpecials(void)
{
	spritetype   *thisSprite;
	jprojectTile *thisSpecial;
	int newpicnum = 0;

	for(int i = 0; i < activeSpecials.size(); i++)
	{
		thisSpecial    = activeSpecials.at(i);
		thisSprite     = build_func->unsafeGetSprite((short)thisSpecial->spritenum);


		if(thisSprite->picnum >= 984 && thisSprite->picnum <= 996)
		{
			newpicnum += ((12+(build_func->ftotalclock()>>3))%10);

			thisSprite->picnum = 984+newpicnum;

			if(thisSprite->picnum <= 984)
			{
				activeSpecials.erase(activeSpecials.begin() + i);
				thisSprite->picnum = 4580;
			}
		}
	}
}

/*
===============================
void Game::DisplayLevel(void)
===============================
*/
void Game::DisplayLevel(void)
{
	bloodPlayer_t *thisPlayer = gamePlayers.at(0);
	long tx, ty, tz;
	short pskyoff[256];     
	if(ingame == FALSE)
		return;

	pskyoff[0]=1; pskyoff[1]=2; pskyoff[2]=1; pskyoff[3]=3;
    pskyoff[4]=4; pskyoff[5]=0; pskyoff[6]=2; pskyoff[7]=3;

	build_func->Justin_SetSky(65536L,pskyoff);

    tx = thisPlayer->oldx + mulscale(thisPlayer->posx - thisPlayer->oldx, 65536, 16);
    ty = thisPlayer->oldy + mulscale(thisPlayer->posy - thisPlayer->oldy, 65536, 16);
    tz = thisPlayer->oldz + mulscale(thisPlayer->posz - thisPlayer->oldz, 65536, 16);
  
	#ifndef FAFTEST
		SectorFOF(tx,ty,tz,thisPlayer->ang,1,65536);
	#else
		FAFDraw(tx,ty,tz,thisPlayer->ang, 70,thisPlayer->sectorNum);
	#endif
	build_func->displayLevel(thisPlayer->posx,thisPlayer->posy,thisPlayer->posz,thisPlayer->ang,thisPlayer->sectorNum,65536);
}

/*
====================================
int Game::RunFrame(int totalclock)
====================================
*/
boolean Game::g_PlayerInGame(void)
{
	return this->ingame;
}

/*
===================================================================
void Game::DrawHudNumber(int number, int startfont, int shade,int x, int y)
===================================================================
*/

void Game::DrawHudNumber(int number, long startfont, int shade, int x, int y)
{
	char thisText[20];
	sprintf(thisText,"%d",number);
	guimanager->DrawTextCustom(thisText,startfont,shade,x,y);
}

/*
=========================================
void Game::ShutdownWeaponThreads(void)

Shutdown ALL weapon threads, regardless
if they are on.  This ensures only one
weapon thread is running, which saves
memory.
=========================================
*/

void Game::ShutdownWeaponThreads(void)
{
	for(int i = 0; i < playerWeapons.size(); i++)
	{
		playerWeapons.at(i)->ShutdownThread();
	}
}

/*
==========================
void Game::DrawHUD(void)
==========================
*/

void Game::DrawHUD(void)
{
	bloodPlayer_t *thisPlayer = gamePlayers.at(0);
	if(ingame == FALSE)
		return;

	if(NodrawWeapon == FALSE)
	{
		if(!(thisWeaponX2 == 0 && thisWeaponY2 == 0 && thisWeaponPic2 == 0))
			build_func->rotatesprite(thisWeaponX2<<16,thisWeaponY2-4<<16,65536L,0,thisWeaponPic2,16,0,10,0,0,800-1,600-1);
		if(!(thisWeaponX == 0 && thisWeaponY == 0 && thisWeaponPic == 0))
			build_func->rotatesprite(thisWeaponX<<16,thisWeaponY-4<<16,65536L,0,thisWeaponPic,16,0,10,0,0,800-1,600-1);
	}
	else
	{
		thisGame->NodrawWeapon = FALSE;
	}
	
	build_func->rotatesprite(160<<16,172<<16,65536L,0,STAUSBAR,16,0,10,0,0,800-1,600-1);
	DrawHudNumber(gamePlayers.at(0)->health,2190,10,85,183);

	DrawHudNumber(0,2240,12,220,183); // current ammo

	// ammo 2-0
	//DrawHudNumber(0,2250,12,137,183); // 0
	//DrawHudNumber(0,2250,12,137,190); // 1
	guimanager->Blood_Printf("alpha4",4417,4400,10,4,9,194);
}

/*
============================
void Game::Shutdown(void)
============================
*/

void Game::Shutdown(void)
{
	printf("Shutting down game\n");
	KillAllThreads();			   // clear all running threads(scripts).
	printf("....clearing game databases\n");
	ai_Shutdown();
	gamePlayers.clear();           // players get clearned on new level
	worldMobs.clear();			   // flush world mobs
	registeredAI.clear();		   // clear registered AI
	worldEvents.clear();		   // clear all world events
	deadSprites.clear();		   // clear list of dead mobs
	soundManager->StopAllsounds();
	soundManager->FlushSystem();
	FSOUND_Close();
}

/*
================================================
void Game::Register_Weapon(char *weaponScript)
================================================
*/

void Game::Register_Weapon(char *weaponScript)
{
	thread_t *thisWeapon = new thread_t(new program_t(weaponScript));
	playerWeapons.push_back(thisWeapon);
}

/*
===================================
void Game::ShotWeapon(int damage)
===================================
*/

void Game::ShotWeapon(int damage)
{

}

/*
===========================================
void Game::SetPlayerWeapon(int weaponNum)
===========================================
*/

void Game::SetPlayerWeapon(int weaponNum)
{
	if(weaponNum < 0 || weaponNum >=playerWeapons.size())
	{
		printf("SetClientWeapon: Range out of bounds %d\n",weaponNum);
		return;
	}
	
	if(weaponInUse == weaponNum)
		return;

	ShutdownWeaponThreads();
	
	// reset the weapon variables
	thisWeaponPic  = 0;
	thisWeaponX    = 0;
	thisWeaponY    = 0;

	thisWeaponPic2 = 0;
	thisWeaponX2   = 0;
	thisWeaponY2   = 0;

//	spriteNum1     = -2; // no assigned ent
	playerWeapons.at(weaponNum)->RunProgram();
	weaponInUse = weaponNum;
}

/*
=====================================
void Game::PauseWeaponThreads(void)
=====================================
*/

void Game::PauseWeaponThreads(void)
{
	//playerWeapons.at(weaponInUse)->ShutdownThread();
}

/*
=====================================
void Game::ResumeWeaponThread(void)
=====================================
*/

void Game::ResumeWeaponThread(void)
{
//	playerWeapons.at(weaponInUse)->RunProgram();
}
/*
====================================
void Game::WriteLog(char *thisTxt)
====================================
*/

void Game::WriteLog(char *thisTxt)
{
	fseek(thislogFile,0,SEEK_END);
	fwrite(thisTxt,1,strlen(thisTxt),thislogFile);
}

/*
==================================
void Game::FatalCrash(char *err)
==================================
*/

void Game::FatalCrash(char *err)
{
	FILE *thisErrFile = fopen("ERROR.TXT","wb");
	fseek(thisErrFile,0,SEEK_SET);
	fwrite(err,1,strlen(err),thisErrFile);
	fclose(thisErrFile);
	fclose(thislogFile);
	exit(0);
}

/*
=========================================
void Game::PlaySfxSprite(short soundid)

Kind of a hacking method for doing this
=========================================
*/

void Game::PlaySfxSprite(short soundid)
{
	switch(soundid)
	{
		case 106:
			soundManager->soundPlay(13);
		break;

		case 157:
			soundManager->soundPlay(14);
		break;
	}
}

/*
===============================
void Game::WorldAmbient(void)
===============================
*/

void Game::WorldAmbient(void)
{
	spritetype    *thisSprite;
	xspritetype   *xsprite;

	for(int thisSfx=0; thisSfx < numofsprites; thisSfx++)
	{
		thisSprite = build_func->Game_GetSprite(thisSfx);
		if(thisSprite->type == 710)
		{
			xsprite = build_func->GetXSprite(thisSfx);
			switch(xsprite->data3)
			{
				case 17:
					soundManager->playsnd3d(thisSfx,15);
				break;
				case 32:
					soundManager->playsnd3d(thisSfx,16);
				break;
				case 35:
					soundManager->playsnd3d(thisSfx,17);
				break;
			}
		}
	}	
}

/*
=============================
void Game::RunEffects(void)
=============================
*/

void Game::RunEffects(void)
{
	
}

/*
============================================
void Game::PlayerSetLocation(int playerNum)
============================================
*/

void Game::PlayerSetLocation(int playerNum)
{
	spritetype *thisSprite;
	int        spriteNum = 0;

	while( 1 )// Loop through all the game sprites
	{
		thisSprite = build_func->Game_GetSprite(spriteNum);

		if(thisSprite == NULL) // reached end of max sprites in level
			break;

		if(thisSprite->picnum == (2522 + playerNum))
		{
			gamePlayers.at(0)->posx = thisSprite->x;
			gamePlayers.at(0)->posy = thisSprite->y;
			gamePlayers.at(0)->posz = thisSprite->z;
		}
		spriteNum++;
	}; 
	this->P_HandleMovement();
}

/*
==================================
void Game::RemoveEffectors(void)
==================================
*/

void Game::RemoveEffectors(void)
{
	spritetype    *thisSprite;
	hiddensprites hideSpr;
	int        spriteNum = 0;
	int        currinvisspr=0;
	numofsprites = 0;
	while( 1 )// Loop through all the game sprites
	{
		thisSprite = build_func->Game_GetSprite(spriteNum);

		if(thisSprite == NULL) // reached end of max sprites in level
			return;

		while( 1 )
		{
			hideSpr = InvisibleSprites[currinvisspr];

			if(hideSpr.endtile == -1 && hideSpr.starttile == -1 &&
			   hideSpr.type    == -1)
			{
			   break;
			}

			if((thisSprite->picnum >= hideSpr.starttile && thisSprite->picnum <= hideSpr.endtile) ||
				thisSprite->type == hideSpr.type)
			{
				thisSprite->cstat = (short)32768;
			}
			currinvisspr++;
		}

		currinvisspr=0;
		numofsprites++;
		spriteNum++;
	}; 
}

/*
====================================================================
void Game::RegisterAI(int startPic, int endPic, char *script_name)
====================================================================
*/

void Game::RegisterAI(int startPic, int endPic, char *script_name)
{
	monster_t *thisMob = new monster_t;
	printf("Registering AI: (%d -> %d) %s\n",startPic,endPic,script_name);

	thisMob->mobAI    = new thread_t(new program_t(script_name));
	thisMob->startPic = startPic;
	thisMob->endPic   = endPic;

	gameAIScripts.push_back(script_name);

	registeredAI.push_back(thisMob);
}

/*
================================================================
void Game::ModifySprite(int spriteNum, spritetype *thisSprite)
================================================================
*/

void Game::ModifySprite(int spriteNum, spritetype *thisSprite)
{
	spritetype *thisSpriteLocal = build_func->Game_GetSprite(spriteNum);
    thisSpriteLocal             = thisSprite;
}

/*
==========================
void Game::StartAI(void)
==========================
*/

void Game::StartAI(void)
{
	monster_t *thisMob;
	spritetype *thisSprite;
	int        spriteNum = 0;

	printf("....running AI threads\n");
	while( 1 )// Loop through all the game sprites
	{
		thisSprite = build_func->Game_GetSprite(spriteNum);

		if(thisSprite == NULL) // reached end of max sprites in level
			return;

		if(registeredAI.size() == 0)
			printf("WARNING: No registered ai?\n");

		for(int i = 0; i < registeredAI.size(); i++)
		{
			thisMob   = new monster_t;

			thisMob->endPic   = registeredAI.at(i)->endPic;
			thisMob->startPic = registeredAI.at(i)->startPic;
			if(thisMob->startPic == 2820 && thisMob->endPic == 2937)
				thisMob->mobAI    = new thread_t(new program_t("monster_ai_cult.c"));
			if(thisMob->startPic == 1170 && thisMob->endPic == 1258)
				thisMob->mobAI    = new thread_t(new program_t("monster_ai_zombie.c"));
			if(thisMob->startPic == 3054 && thisMob->endPic == 3054)
				thisMob->mobAI    = new thread_t(new program_t("monster_ai_zombie.c"));

            if(thisSprite->picnum >= thisMob->startPic && thisSprite->picnum <= thisMob->endPic)
			{
				monsterEnt_t *thisEnt = new monsterEnt_t;
	
				thisEnt->thisMob = thisMob;
				thisEnt->mobID   = spriteNum;
				thisEnt->aiNum   = i;
				thisEnt->thisMob->mobAI->thisEntNum = spriteNum;  
				thisEnt->thisMob->mobAI->RunProgram();

				worldMobs.push_back(thisEnt); // keep track of our entities
				break;
			}
		}
		spriteNum++;
	}; 
}

/*
=============================
void Game::ShutdownAI(void)
=============================
*/

void Game::ShutdownAI(void)
{
	// Stop all AI threads
	printf("Game: Shutting down AI threads FIXME\n");

	for(int i = 0; i < worldMobs.size(); i++)
	{
		worldMobs.at(i)->thisMob->mobAI->ShutdownThread();
	}
	registeredAI.clear();
}

/*
=============================
void Game::ShutdownAI(void)
=============================
*/

void Game::SuspendAI(void)
{
	// Stop all AI threads
	for(int i = 0; i < worldMobs.size(); i++)
	{
		worldMobs.at(i)->thisMob->mobAI->ShutdownThread();
	}
}

/*
=============================
void Game::ShutdownAI(void)
=============================
*/

void Game::ReRunAI(void)
{
	// Stop all AI threads
	for(int i = 0; i < worldMobs.size(); i++)
	{
		worldMobs.at(i)->thisMob->mobAI->RunProgram();
	}
}

/*
==============================
void Game::DrawAutoMap(void)

Info:
Draws a 2D map of the level onto of view.

Author:
3Drealms Duke3d Source Code.  Modified
for WinBlood.
==============================
*/

void Game::DrawAutoMap(void)
{
	bloodPlayer_t *thisPlayer = gamePlayers.at(0);
	walltype      *thisWall,*thisWall2;
	int           wallNum;
	long		  k;
	long startWall, endWall;

	long thisX    = thisPlayer->posx;
	long thisY    = thisPlayer->posy;
	long thisZ    = thisPlayer->posz;
	long thisAng  = thisPlayer->ang;
	long thisSect = thisPlayer->sectorNum;

	long ox,oy,x1,y1,x2,y2,xvect,yvect,xvect2,yvect2;
	long cosa, sina,daang;
	long ang,cang;

	sina = (sin((long double)thisPlayer->ang * (3.141592/1024)) * 16383);
	cosa = (cos((long double)thisPlayer->ang * (3.141592/1024)) * 16383);

	xvect = sina * 450;
    yvect = cosa * 450;
    xvect2 = mulscale16(xvect,65536);
    yvect2 = mulscale16(yvect,65536);

	if(numSectors <= 0)
		printf("WARNING: DrawAutoMap, no sectors in map...!!?@?#?\n");

	guimanager->Blood_Printf(this->thisBoard,4417, 4400,4,5,20,160);
	guimanager->Blood_Printf("winblood",4417, 4400,4,5,20,170);
	// Draw the white lines, drawing red ones requires a extra few steps
	// but I think white ones will work just fine
    for(int i = 0; i < this->numSectors; i++)
	{
		startWall = build_func->GetSector(i)->wallptr;
		endWall   = build_func->GetSector(i)->wallptr + build_func->GetSector(i)->wallnum;
		
		wallNum = startWall;
		k = -1;
		for(int j = startWall; j < endWall; j++,wallNum++)
		{
			thisWall = build_func->get_Wall(wallNum);
            k = thisWall->nextwall;

            if (build_func->getTileX(thisWall->picnum) == 0) continue;
            if (build_func->getTileY(thisWall->picnum) == 0) continue;

            if(k >= 0)
				continue;
			
             ox = thisWall->x-thisPlayer->posx; oy = thisWall->y-thisPlayer->posy;
             x1 = dmulscale16(ox,xvect,-oy,yvect)+(800<<11);
             y1 = dmulscale16(oy,xvect2,ox,yvect2)+(160<<11);   

			 k = thisWall->point2; thisWall2 = build_func->get_Wall(k);
             ox = thisWall2->x-thisPlayer->posx; oy = thisWall2->y-thisPlayer->posy;
             x2 = dmulscale16(ox,xvect,-oy,yvect)+(800<<11);
             y2 = dmulscale16(oy,xvect2,ox,yvect2)+(160<<11);

             build_func->drawline256(x1,y1,x2,y2,24);
		}
	}

	// Only draw this player, rest is just cheating : ).

	ox = thisPlayer->posx; oy = thisPlayer->posy;
    daang = (thisPlayer->ang)&2047;
    x1 = mulscale(ox,xvect,16) - mulscale(oy,yvect,16);
    y1 = mulscale(oy,xvect2,16) + mulscale(ox,yvect2,16);
#if 0
	build_func->rotatesprite((x1<<4)+(800<<15),(y1<<4)+(600<<15),65536L,
                daang,3837,7,1,
                10,0,0,800-1,400-1);
#endif
}

/*
===================================
void Game::Level_MakeActors(void)

Info:
Not used
===================================
*/

void Game::Level_MakeActors(void)
{
	// Regiser ALL actors that are in game.
//	bloodEntity_t *monster_CultGuy = new bloodEntity_t(2820,90,-1,-1
//													   2860,
}

/*
==================================
void Game::CheckExpansions(void)
==================================
*/

void Game::CheckExpansions(void)
{
	FILE *testMap;
	//expansionLists

	testMap = fopen("e2m1.map","rb");

	if(!testMap) // demo
	{
		printf("DEMO VERSION.  PLEASE BUY THE FULL GAME\n");
		expansionLists = -2;
		fclose(testMap);
		return;
	}
	fclose(testMap);
	printf("RETAIL VERSION - DO NOT DISTRIBUTE\n");

	testMap = fopen("e6m1.map","rb");
	if(testMap)
	{
		printf("PLASMA PACK EXPANSION ENABLED\n");
		expansionLists = 1; // plasma pack
	    fclose(testMap);    // jm: - fixed crash March 27th 2005
	}
}

/*
============================
void Game::CheckArgs(void)
============================
*/

void Game::CheckArgs(void)
{
	char *thisParam;
	int  i = 0;
	
	thisParam = build_func->getParam(0);
	while(thisParam != NULL)
	{
		if(!strcmp(thisParam,"+map"))
		{
			i++;
			printf("Found map arg %s\n",build_func->getParam(i));
		}
		i++;
		thisParam = build_func->getParam(i);
	}
}

/*
====================================
int Game::RunFrame(int totalclock)
====================================
*/

int Game::RunFrame(int totalclock)
{
	long globhiz=0, globhihit=0, globloz=0,globlohit=0;
	long goalz;
	float posupdate[3];
	bloodPlayer_t  *thisPlayer;

	if(initframe != 1)
	{
		int  menuMusic;
		ReadBloodCFG();
		srand(time(NULL));
		init_system();
	//	CheckArgs();
		menuMusic = soundManager->cacheMusic("PESTIS");
		initframe = 1;
		CheckExpansions();
		soundManager->PlayMusic(menuMusic);
		menuManager->SetMenu(BLOODMENU_ESC);
	}

	if(gamePlayers.size() > 0)
		thisPlayer = gamePlayers.at(0);

	if(exit_game)
	{
		Shutdown();
		return 0;
	}

	if(build_func->build_KeyDown(sc_Q))      // quick escape
	{
		Shutdown();
		return 0;
	}

	if(build_func->build_KeyDown(sc_Escape)) // menu key is hard coded
	{
		autoMap = FALSE;
		PauseWeaponThreads();
		SuspendAI();
		menuManager->SetMenu(BLOODMENU_ESC);
	}

	if(consoleactive == TRUE)
	{
		if(build_func->build_KeyDown(0x29))
		{
			consoleactive = FALSE;
			build_func->build_ClearKey(0x29);
		}
		else
		{
			build_func->nextpage();
			console->DrawConsole(120);
		}
	}

	if(ingame == TRUE && consoleactive == FALSE)
	{
		if(menuManager->MenuActive() == FALSE && gamePlayers.size() > 0)
		{
			if(thisPlayer->health <= 0) // dead people can't move
			{
				build_func->getzrange(thisPlayer->posx,thisPlayer->posy,thisPlayer->posz,thisPlayer->sectorNum,&globhiz,&globhihit,&globloz,&globlohit,128L,CLIPMASK0);

				thisPlayer->posz = globloz;
				skippageflip = 1;
			}
		}
#if 0
		if(build_func->build_KeyDown(0x29))
		{
			consoleactive = TRUE;
			build_func->build_ClearKey(0x29);
		}
#endif
		// Really nasty hack
		if(multiplayer == TRUE && this->registeredAI.size() > 0)
		{
			// remove all the entitys that where registered and flag them
			for(int i = 0; i < registeredAI.size(); i++)
			{
				for(int j = 0; j < numofsprites; j++)
				{
					spritetype *thisSprite = build_func->Game_GetSprite(j);

					if(thisSprite->picnum >= registeredAI.at(i)->startPic &&
					   thisSprite->picnum <= registeredAI.at(i)->endPic)
					{
							thisSprite->cstat = (short)32768;
					}
				}
			}
			registeredAI.clear();
		}
		if(build_func->build_KeyDown(sc_Tab))
		{
			switch(autoMap)
			{
				case FALSE:
				//	guimanager->Blood_PrintfTimed("Welcome to blood",3000);
					autoMap = TRUE;
				break;

				case TRUE:
					autoMap = FALSE;
				break;
			};
			build_func->build_ClearKey(sc_Tab);
		}

		if(autoMap == TRUE)
			DrawAutoMap();
		if(build_func->build_KeyDown(sc_Space))      // sprites, doors are handled differentlly
			OperateSprites();

		OperateSpritesAuto();
		OperateSectors();
		guimanager->Init2D();    // this always needs to be first call

		WorldAmbient();
		RunOperators();
		RunEffects();

		DisplayLevel();			 // draw the level

		if(menuManager->MenuActive() == TRUE)
			P_HandleMovement();      // move the player position for this frame
		DrawHUD();				 // draw the players hud
		RunProjectiles();
		RunSpecials();
#if 0
		posupdate[0] = thisPlayer->posx;
		posupdate[1] = thisPlayer->posy;
		posupdate[2] = thisPlayer->posz;
		FSOUND_3D_Listener_SetAttributes(posupdate,NULL,NULL,NULL,NULL,NULL,NULL,NULL);
		FSOUND_Update();
#endif
	}

	
	menuManager->DrawMenu(); // menu gets drawn before everything else except the loading screen

	// update sin table
	//build_func->sintable = build_func->getSinTable();

	if(menuManager->MenuActive() == FALSE)
	{
		if(skippageflip != 1)
			build_func->nextpage(); // this is always the last call, except in menus
	}

	return 1; // frame is OK
}