//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

PORTIONS OF CODE FROM DUKE NUKEM 3D SOURCE CODE
*/
//-------------------------------------------------------------------------

// game_menu.cpp

#include "game.h"

char *gameMenu_t::VideoTypes[] = { "normal 8bit",
								   "normal 16bit",
								   "polymost 16bit",
								   "polymost 32bit" };

char *gameMenu_t::resolution[] = { "320x200",
								   "640x480",
								   "800x600",
								   "1024x768",
								   "1280x1024" };

int gameMenu_t::resBpp[]       = { 8,
								   16,
								   16,
								   32 };

int gameMenu_t::resWidth[]     = { 320,
								   640,
								   800,
								   1024,
								   1280 };

int gameMenu_t::resHeight[]    = { 200,
								   480,
								   600,
								   768,
								   1024 };


char *gameMenu_t::mboolType[]  = { "no",
								   "yes" };

gameMenu_t::gameMenu_t()
{
	bloodDrips *blankDrip = new bloodDrips;

	currentMenu = BLOODMENU_OFF;
	for(int i = 0; i < 7; i++)
	{
		menuDrips.push_back(blankDrip);
	}

	lobbyUsers.push_back("blinkskater");

	tmpNum            = 0;
	handleEscape      = false;
	lastconnectfailed = false;
}
/*
====================================================================================
void gameMenu_t::ChangeResolution(int resolution, int antistrophy, int fullscreen,
								  int renderer)
====================================================================================
*/
void gameMenu_t::ChangeResolution(int resolution, int antistrophy, int fullscreen,
								  int renderer)
{
	int glBased = 0;
	if(resolution > 1)
		glBased = 1;
	build_func->Change_ScreenInfo(resWidth[resolution], resHeight[resolution], resBpp[resolution],
								  antistrophy, glBased ,fullscreen);
}

/*
=======================================
void gameMenu_t::SetMenu(int menuID)
=======================================
*/

void gameMenu_t::SetMenu(int menuID)
{
	// Menu can only be switched off in game
/*
	if(thisGame->g_PlayerInGame() != true && MenuActive)
	{
		currentMenu = BLOODMENU_OFF;
		return;
	}
*/
	if(menuID == BLOODMENU_ESC)
	{
		handleEscape = true;
		//menuID = BLOODMENU_MAIN;
		if(currentMenu > 0)
			return;
	}
	//if(currentMenu > 0)
	//	lastMenu     = currentMenu;

	currentMenu  = menuID;
	FlushMenu();
	slot = 0;
	build_func->nextpage();
}

/*
==================================
void gameMenu_t::FlushMenu(void)
==================================
*/

void gameMenu_t::FlushMenu(void)
{
	currentMenuItems.clear();
	thisMenuFunc.clear();

	if(currentMenuItems.empty() == false)
		printf("WARNING: Unable to clear menu buffer\n");
}

/*
=======================================================
void gameMenu_t::AddItemToMenu(menuItems_t *thisItem)
=======================================================
*/

void gameMenu_t::AddItemToMenu(menuItems_t *thisItem)
{
	currentMenuItems.push_back(thisItem);
}

/*
======================================
void gameMenu_t::DrawThisMenu(void)
======================================
*/

void gameMenu_t::ThisMenuMain(void)
{
	if(build_func->build_KeyDown(sc_DownArrow))
	{
		slot++;
		if(slot > currentMenuItems.size()-1)
			slot = 0;
	}
	if(build_func->build_KeyDown(sc_UpArrow))
	{
		slot--;
		if(slot < 0)
			slot = currentMenuItems.size()-1;
	}

	for(int i = 0; i < currentMenuItems.size(); i+=1)
	{
		if(slot == i)
			guimanager->DrawTextLight((char *)currentMenuItems.at(i)->name.c_str(),fontNum,currentMenuItems.at(i)->x,currentMenuItems.at(i)->y);
		else
			guimanager->DrawText((char *)currentMenuItems.at(i)->name.c_str(),fontNum, currentMenuItems.at(i)->x,currentMenuItems.at(i)->y);
	//	printf("%s\n",(char *)currentMenuItems.at(i)->name.c_str());
	}
}

/*
===================================
void gameMenu_t::menu_quit(void)
===================================
*/

void gameMenu_t::menu_quit(void)
{
	thisGame->exit_game = true;
}

/*
=======================================
void gameMenu_t::menu_diffculty(void)
=======================================
*/

void gameMenu_t::menu_diffculty(void)
{
	menuItems_t *diff1	   = CreateBloodMenu("Still Kicking",NULL,93,50);
	menuItems_t *diff2	   = CreateBloodMenu("Pink on the Inside",NULL,73,70);
	menuItems_t *diff3	   = CreateBloodMenu("Lightly Broiled",NULL,73,90);
	menuItems_t *diff4	   = CreateBloodMenu("Well done",NULL,110,110);
	menuItems_t *diff5	   = CreateBloodMenu("Extra Crispy",NULL,103,130);

	fontNum = 4257;

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("DIFFICULTY",113,20);

	if(build_func->build_KeyDown(sc_Enter))
	{
		thisGame->multiplayer = false;
		this->pLevel = 0;
		switch(this->pEpisode)
		{
			case 0:
				if(thisGame->expansionLists != -2)
					thisGame->LoadBoard("e1m1.map");
			break;
			case 1:
				if(thisGame->expansionLists != -2)
					thisGame->LoadBoard("e2m1.map");
			break;
			case 2:
				//if(thisGame->expansionLists != -2)
					//thisGame->LoadBoard("e3m1.map");
			break;
			case 3:
				if(thisGame->expansionLists != -2)
					thisGame->LoadBoard("e4m1.map");
			break;

			case 4:
				if(thisGame->expansionLists == 1)
					thisGame->LoadBoard("E6M1.MAP");
			break;

		}
		SetMenu(BLOODMENU_OFF);
		return;
	}
	lastMenu = BLOODMENU_EPISODE;

	AddItemToMenu(diff1);
	AddItemToMenu(diff2);
	AddItemToMenu(diff3);
	AddItemToMenu(diff4);
	AddItemToMenu(diff5);
}


/*
=======================================
void gameMenu_t::Menu_UserMaps(void)
=======================================
*/

void gameMenu_t::Menu_UserMaps(void)
{
	menuItems_t *umapSetOption    = CreateBloodMenu("usermap",NULL,20,50);
	menuItems_t *LoadUMapOption   = CreateBloodMenu("load map",NULL,100,80);

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("usermaps",113,20);

	if(slot == 0)
	{
		if(usermapType.size() < 8)
		{
			if(build_func->build_KeyDown(sc_0))
				usermapType.append("0");
			if(build_func->build_KeyDown(sc_1))
				usermapType.append("1");
			if(build_func->build_KeyDown(sc_2))
				usermapType.append("2");
			if(build_func->build_KeyDown(sc_3))
				usermapType.append("3");
			if(build_func->build_KeyDown(sc_4))
				usermapType.append("4");
			if(build_func->build_KeyDown(sc_5))
				usermapType.append("5");
			if(build_func->build_KeyDown(sc_6))
				usermapType.append("6");
			if(build_func->build_KeyDown(sc_7))
				usermapType.append("7");
			if(build_func->build_KeyDown(sc_8))
				usermapType.append("8");
			if(build_func->build_KeyDown(sc_9))
				usermapType.append("9");

			if(build_func->build_KeyDown(sc_A))
				usermapType.append("a");
			if(build_func->build_KeyDown(sc_B))
				usermapType.append("b");
			if(build_func->build_KeyDown(sc_C))
				usermapType.append("c");
			if(build_func->build_KeyDown(sc_D))
				usermapType.append("d");
			if(build_func->build_KeyDown(sc_E))
				usermapType.append("e");
			if(build_func->build_KeyDown(sc_F))
				usermapType.append("f");
			if(build_func->build_KeyDown(sc_G))
				usermapType.append("g");
			if(build_func->build_KeyDown(sc_H))
				usermapType.append("h");
			if(build_func->build_KeyDown(sc_I))
				usermapType.append("i");
			if(build_func->build_KeyDown(sc_J))
				usermapType.append("j");
			if(build_func->build_KeyDown(sc_K))
				usermapType.append("k");
			if(build_func->build_KeyDown(sc_L))
				usermapType.append("l");
			if(build_func->build_KeyDown(sc_M))
				usermapType.append("m");
			if(build_func->build_KeyDown(sc_N))
				usermapType.append("n");
			if(build_func->build_KeyDown(sc_O))
				usermapType.append("o");
			if(build_func->build_KeyDown(sc_P))
				usermapType.append("p");
			if(build_func->build_KeyDown(sc_Q))
				usermapType.append("q");
			if(build_func->build_KeyDown(sc_R))
				usermapType.append("r");
			if(build_func->build_KeyDown(sc_S))
				usermapType.append("s");
			if(build_func->build_KeyDown(sc_T))
				usermapType.append("t");
			if(build_func->build_KeyDown(sc_U))
				usermapType.append("u");
			if(build_func->build_KeyDown(sc_V))
				usermapType.append("v");
			if(build_func->build_KeyDown(sc_W))
				usermapType.append("w");
			if(build_func->build_KeyDown(sc_X))
				usermapType.append("x");
			if(build_func->build_KeyDown(sc_Y))
				usermapType.append("y");
			if(build_func->build_KeyDown(sc_Z))
				usermapType.append("z");
		}

		if(build_func->build_KeyDown(sc_Enter) && usermapType.size() > 0)
			usermapType.clear();

		build_func->KBClearLastScanCode();
	}

	if(build_func->build_KeyDown(sc_Enter) && slot == 1)
	{
		usermapType.append(".map");
		this->pDiffuculty = 1;
		this->pEpisode    = 1;
		this->pLevel      = 1;
		thisGame->LoadBoard("e1m6.map");
		SetMenu(BLOODMENU_OFF);
		return;
	}
	fontNum  = 4257;
	lastMenu = BLOODMENU_EPISODE;

	guimanager->Blood_Printf((char *)usermapType.c_str(),4417, 4400,4,5,220,50);
	AddItemToMenu(umapSetOption);
	AddItemToMenu(LoadUMapOption);
}

/*
======================================
void gameMenu_t::menu_saveGame(void)
======================================
*/

void gameMenu_t::menu_saveGame(void)
{
	FILE        *thisSave[8];
	menuItems_t *saveSlots[8];
	char        *tempBuf = new char[20];
	char		savName[20];

	for(int i = 0; i < 8; i++)
	{
		sprintf(tempBuf,"blood00%d.sav",i);
		thisSave[i]   = fopen(tempBuf,"rb");

		if(thisSave[i] == NULL)
		{
			sprintf(savName,"<save game>");
		}
		else
		{

		}

		saveSlots[i]  = CreateBloodMenu(savName,NULL,160,50 + (i * 10));
	}

	fontNum = 4161;

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("SAVE GAME",116,20);

	lastMenu = BLOODMENU_MAIN;

	if(build_func->build_KeyDown(sc_Enter))
	{
		saveGame_t *thisSaveGame;

		sprintf(tempBuf,"blood00%d.sav",slot);
		thisSave[slot]   = fopen(tempBuf,"wb");
		sprintf(tempBuf,"Level %s Time %d:%d",thisGame->thisBoard,1,1);

		if(thisSave[slot] == NULL)
		{
			printf("WARNING: Failed to open save game for writing %s\n",tempBuf);
			return;
		}

		thisSaveGame = new saveGame_t(tempBuf,10,100,1,0);

		fseek(thisSave[slot],0,SEEK_SET);
		fwrite(thisSaveGame,sizeof(saveGame_t),1,thisSave[slot]);
        fclose(thisSave[slot]);

	}

	for(int i = 0; i < 8; i++)
	{
		AddItemToMenu(saveSlots[i]);
		if(thisSave[slot] != NULL)
			fclose(thisSave[slot]);
	}
}

/*
=====================================
void gameMenu_t::menu_options(void)
=====================================
*/

#if 0
void gameMenu_t::menu_options(void)
{
	menuItems_t *gammaOption   = CreateBloodMenu("gamma",NULL,60,50);

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("OPTIONS",117,20);

	if(build_func->build_KeyDown(sc_RightArrow))
	{
		if(thisGame->brightness <= 50)
		{
			thisGame->brightness++;
			build_func->Set_Brightness(thisGame->brightness);
			build_func->build_ClearKey(sc_RightArrow);
		}
	}
	else if(build_func->build_KeyDown(sc_LeftArrow))
	{
		if(thisGame->brightness >= 10)
		{
			thisGame->brightness--;
			build_func->Set_Brightness(thisGame->brightness);
			build_func->build_ClearKey(sc_RightArrow);
		}
	}

	// REALLY UGLY CODING FIX ME
	thisGame->DrawHudNumber(thisGame->brightness,4400,7,220,50);

	fontNum  = 4417;
	lastMenu = BLOODMENU_MAIN;

	AddItemToMenu(gammaOption);
}
#else
void gameMenu_t::menu_options(void)
{
	menuItems_t *gameOptions	       = CreateBloodMenu("game options",NULL,102,50);
	menuItems_t *videoOptions	       = CreateBloodMenu("video options",NULL,98,70);
	menuItems_t *soundOptions	       = CreateBloodMenu("sound options",NULL,98,90);
	menuItems_t *KeyboardOptions	   = CreateBloodMenu("keyboard",NULL,110,110);

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("OPTIONS",117,20);

	lastMenu = BLOODMENU_MAIN;
	fontNum = 4257;

	if(build_func->build_KeyDown(sc_Enter))
	{
		switch(slot)
		{
			case 1:
				SetMenu(BLOODMENU_VIDEO);
			break;
		}
	}

	AddItemToMenu(gameOptions);
	AddItemToMenu(videoOptions);
	AddItemToMenu(soundOptions);
	AddItemToMenu(KeyboardOptions);
}
#endif

/*
===================================
void gameMenu_t::menu_video(void)
===================================
*/

void gameMenu_t::menu_video(void)
{
	char tempTxt[100];
	menuItems_t *videoOption = CreateBloodMenu("build renderer",NULL,20,50);
	menuItems_t *antiOption  = CreateBloodMenu("anisotropy",NULL,20,70);
	menuItems_t *ResOption   = CreateBloodMenu("resolution",NULL,20,90);
	menuItems_t *fsOption    = CreateBloodMenu("fullscreen",NULL,20,110);
	menuItems_t *ApplyOption = CreateBloodMenu("apply changes",NULL,100,140);

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash(" VIDEO",117,20);

	guimanager->Blood_Printf(VideoTypes[thisGame->thisRenderer],
							 4417, 4400,4,5,220,50);

	sprintf(tempTxt,"%d",thisGame->anisotropy);
	if(thisGame->anisotropy > 1)
	{
		guimanager->Blood_Printf(tempTxt, 4417, 4400,4,5,220,70);
	}
	else
	{
		guimanager->Blood_Printf("none", 4417, 4400,4,5,220,70);
	}

	guimanager->Blood_Printf(resolution[thisGame->resolution],
							 4417, 4400,4,5,220,90);

	guimanager->Blood_Printf(mboolType[thisGame->fullscreen],
							 4417, 4400,4,5,220,110);

	if(build_func->build_KeyDown(sc_Enter))
	{
		switch(slot)
		{
			case 0:
				thisGame->thisRenderer++;
				if(thisGame->thisRenderer > 3)
					thisGame->thisRenderer = 0;
				build_func->build_ClearKey(sc_Enter);
			break;

			case 1:
				thisGame->anisotropy++;
				if(thisGame->anisotropy > 8)
					thisGame->anisotropy = 1;
				build_func->build_ClearKey(sc_Enter);
			break;
			case 2:
				thisGame->resolution++;
				if(thisGame->resolution > 4)
					thisGame->resolution = 0;
				build_func->build_ClearKey(sc_Enter);
			break;
			case 3:
				thisGame->fullscreen++;
				if(thisGame->fullscreen > 1)
					thisGame->fullscreen = 0;
				build_func->build_ClearKey(sc_Enter);
			break;
			case 4:
				gameCFG_t *newCfg = new gameCFG_t;
				newCfg->antistrophy = thisGame->anisotropy;
				newCfg->fullscreen  = thisGame->fullscreen;
				newCfg->renderer    = thisGame->thisRenderer;
				newCfg->resolution  = thisGame->resolution;
				ChangeResolution(newCfg->resolution,newCfg->antistrophy,newCfg->fullscreen,
								 newCfg->renderer);
				WriteBloodCFG(newCfg);
			break;
		}
	}

	fontNum  = 4257;
	lastMenu = BLOODMENU_OPTIONS;

	AddItemToMenu(videoOption);
	AddItemToMenu(antiOption);
	AddItemToMenu(ResOption);
	AddItemToMenu(fsOption);
	AddItemToMenu(ApplyOption);
}

/*
====================================
void gameMenu_t::menu_episode(void)
====================================
*/

void gameMenu_t::menu_episode(void)
{
	menuItems_t *episode1	   = CreateBloodMenu("The way of all flesh",NULL,60,50);
	menuItems_t *episode2	   = CreateBloodMenu("Even death may die",NULL,70,70);
	menuItems_t *episode3	   = CreateBloodMenu("Farewell to arms",NULL,82,90);
	menuItems_t *episode4	   = CreateBloodMenu("Dead Reckoning",NULL,89,110);
	menuItems_t *episode5	   = CreateBloodMenu("Post Mortem",NULL,98,130);
	menuItems_t *useropt	   = CreateBloodMenu("user maps",NULL,98,150);

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("EPISODE",126,20);

	if(build_func->build_KeyDown(sc_Enter))
	{
		if(slot == 5) // user maps
		{
			SetMenu(BLOODMENU_UMAPS);
			return;
		}
		char temp[40];
		this->pEpisode = slot;
		sprintf(temp,"CBLOOD%d",this->pEpisode+1);
		thisGame->levelMusic = soundManager->cacheMusic(temp);
		SetMenu(BLOODMENU_DIFFUC);
		return;
	}

	fontNum = 4257;

	lastMenu = BLOODMENU_MAIN;

	AddItemToMenu(episode1);
	AddItemToMenu(episode2);
	AddItemToMenu(episode3);
	AddItemToMenu(episode4);

	if(thisGame->expansionLists == 1)
		AddItemToMenu(episode5);

	if(thisGame->expansionLists != -2)
		AddItemToMenu(useropt);
}

/*
=================================
void gameMenu_t::menu_main(void)
=================================
*/

void gameMenu_t::menu_main(void)
{
	menuItems_t *newGame         = CreateBloodMenu("NEW GAME",NULL,115,50);
	menuItems_t *multiplayer     = CreateBloodMenu("MULTIPLAYER",NULL,100,70);
	menuItems_t *options         = CreateBloodMenu("OPTIONS",NULL,120,90);
	menuItems_t *loadgame        = CreateBloodMenu("LOAD GAME",NULL,109,110);
	menuItems_t *savegame        = CreateBloodMenu("SAVE GAME",NULL,110,130);
	menuItems_t *help            = CreateBloodMenu("HELP",NULL,132,130);
	menuItems_t *credits         = CreateBloodMenu("CREDITS",NULL,120,150);
	menuItems_t *quit            = CreateBloodMenu("QUIT",NULL,132,170);

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("BLOOD",130,20);

	fontNum = 4257;

	if(build_func->build_KeyDown(sc_Enter))
	{
		if(slot == 0)
			SetMenu(BLOODMENU_EPISODE);

		#ifdef MULTIPLAYER_TEST
			if(slot == 1)
				SetMenu(BLOODMENU_MULTI);
		#endif

		if(slot == 2)
			SetMenu(BLOODMENU_OPTIONS);
		//if(slot == 4 && thisGame->g_PlayerInGame() == true)
		//	SetMenu(BLOODMENU_SAVEG);
		if(slot == 6)
			menu_quit();

		return;
	}

	//if(currentMenuItems.size() > 0)
	//	return;

	AddItemToMenu(newGame);
	AddItemToMenu(multiplayer);
	AddItemToMenu(options);
	AddItemToMenu(loadgame);
	if(thisGame->g_PlayerInGame() == false)
		AddItemToMenu(help);
	else
		AddItemToMenu(savegame);
	AddItemToMenu(credits);
	AddItemToMenu(quit);
}


/*
======================================
void gameMenu_t::DrawBloodDrips(void)
======================================
*/

void gameMenu_t::DrawBloodDrips(void)
{
	int bloodState = ((6+(build_func->ftotalclock()>>3))%7);
	int bloodTile = MENUBLOOD+6-bloodState;
#if 0
	// Reset the drips on change frame

	switch(bloodState)
	{
		case 6:
			menuDrips.at(6)->x = 20;
			menuDrips.at(6)->y = 0;
		break;
		case 5:
			menuDrips.at(5)->x = 50;
			menuDrips.at(5)->y = 0;
		break;
		case 4:
			menuDrips.at(4)->x = 240;
			menuDrips.at(4)->y = 0;
		break;
		case 3:
			menuDrips.at(3)->x = 300;
			menuDrips.at(3)->y = 0;
		break;
		case 2:
			menuDrips.at(2)->x = 170;
			menuDrips.at(2)->y = 0;
		break;
		case 1:
			menuDrips.at(1)->x = 130;
			menuDrips.at(1)->y = 0;
		break;
		case 0:
			menuDrips.at(0)->x = 200;
			menuDrips.at(0)->y = 0;
		break;
	}

	//for(int i = 0; i < 7; i++)
	//{
		// Draw the blood drips
		menuDrips.at(tmpNum)->y+=30;
		//printf("Creating drops at %d: %d %d\n",tmpNum,menuDrips.at(tmpNum)->x,menuDrips.at(tmpNum)->y);
		build_func->rotatesprite(menuDrips.at(tmpNum)->x<<16,menuDrips.at(tmpNum)->y<<16,65536L,0,2028,16,0,10,0,0,800-1,600-1);
	//}
	tmpNum++;
	if(tmpNum > 6)
		tmpNum = 0;
#endif
	build_func->rotatesprite(160<<16,5<<16,65536L,0,bloodTile,4,0,10,0,0,800-1,600-1);
	//build_func->Set_Brightness(30);
	//build_func->nextpage();
}

/*
======================================
boolean gameMenu_t::MenuActive(void)
======================================
*/

boolean gameMenu_t::MenuActive(void)
{
	if(currentMenu == BLOODMENU_OFF)
		return true;
	else
		return false;
}


/*
===================================
void gameMenu_t::Menu_Lobby(void)
===================================
*/

void gameMenu_t::Lobby_PrintNames(void)
{
	for(int i = 0; i < lobbyUsers.size(); i++)
	{
		guimanager->Blood_Printf((char *)lobbyUsers.at(i).c_str(),4417,4496,9,5,255,20);
	}
}


void gameMenu_t::Menu_Lobby(void)
{
	guimanager->DrawTile(4575,0,0,320,200);
	lastMenu = BLOODMENU_MULTI;

	netManager->netinit(); // this in reality is only called once

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("LOBBY",130,20);

	Lobby_PrintNames();
}
/*
=========================================
void gameMenu_t::Menu_JoinGame(void)
=========================================
*/

void gameMenu_t::Menu_JoinGame(void)
{
	menuItems_t *ipOption      = CreateBloodMenu("server ip",NULL,20,50);
	menuItems_t *connectOption = CreateBloodMenu("connect",NULL,100,70);

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("JOIN GAME",100,20);

	if(lastconnectfailed == true)
		guimanager->Blood_Printf("failed to connect",4257,-1,5,10,20,150);

	fontNum  = 4257;
	lastMenu = BLOODMENU_MULTI;

	if(build_func->build_KeyDown(sc_Enter))
	{
		switch(slot)
		{
			case 1:
				lastconnectfailed = false; // reset it
				if(!netManager->netConnect("192.168.1.224"))
				{
					lastconnectfailed = true;
				}
				else
				{
					lastconnectfailed = false;
				}
			break;
		}
	}

	AddItemToMenu(ipOption);
	AddItemToMenu(connectOption);
}

/*
=========================================
void gameMenu_t::Menu_Multiplayer(void)
=========================================
*/

void gameMenu_t::Menu_Multiplayer(void)
{
	menuItems_t *joinG             = CreateBloodMenu("JOIN A GAME",NULL,96,50);
	menuItems_t *startG            = CreateBloodMenu("START A GAME",NULL,92,70);
	menuItems_t *setupG            = CreateBloodMenu("PLAYER SETUP",NULL,92,90);

	build_func->rotatesprite(148<<16,20<<16,65536L,0,2038,4,0,10,0,0,800-1,600-1);
	guimanager->DrawTextNoFlash("MULTIPLAYER",100,20);

	fontNum = 4257;
	lastMenu = BLOODMENU_MAIN;

	lastconnectfailed = false;

	if(build_func->build_KeyDown(sc_Enter))
	{
		switch(slot)
		{
			case 0:
				thisGame->levelMusic = soundManager->cacheMusic("CBLOOD1");
				netManager->netinit();
				SetMenu(BLOODMENU_MJOIN);
			break;
			case 1:

				this->pEpisode = 0; // default for multiplayer/user maps
				thisGame->levelMusic = soundManager->cacheMusic("CBLOOD1");
				thisGame->multiplayer = true;
				SetMenu(BLOODMENU_OFF);
				netManager->netinit();
				netManager->netServerInit();
				thisGame->LoadBoard("e1m1.map");

				return;
			break;
		}
	}

	AddItemToMenu(joinG);
	AddItemToMenu(startG);
	AddItemToMenu(setupG);
}

/*
================================
void gameMenu_t::DrawMenu(void)
================================
*/
void gameMenu_t::DrawMenu(void)
{
	if(currentMenu == BLOODMENU_OFF) // No menu to display.
		return;

	if(thisGame->g_PlayerInGame() == false) // don't force the background in game
		guimanager->DrawTile(MENUBK,0,0,320,200);

	if(handleEscape)
	{
		if(currentMenu != BLOODMENU_MAIN)
		{
			if(currentMenu != BLOODMENU_ESC)
				SetMenu(this->lastMenu);
			else
				SetMenu(BLOODMENU_MAIN);
		}
		else
		{
			if(thisGame->g_PlayerInGame() == true)
			{
				SetMenu(BLOODMENU_OFF);
				thisGame->ResumeWeaponThread();
				thisGame->ReRunAI();
			}
		}
		handleEscape = false;
	}

	if(currentMenu == BLOODMENU_MAIN)
	{
		menu_main();
	}
	else if(currentMenu == BLOODMENU_EPISODE)
	{
		menu_episode();
	}
	else if(currentMenu == BLOODMENU_DIFFUC)
	{
		menu_diffculty();
	}
	else if(currentMenu == BLOODMENU_SAVEG)
	{
		menu_saveGame();
	}
	else if(currentMenu == BLOODMENU_OPTIONS)
	{
		menu_options();
	}
	else if(currentMenu == BLOODMENU_VIDEO)
	{
		menu_video();
	}
	else if(currentMenu == BLOODMENU_MULTI)
	{
		Menu_Multiplayer();
	}
	else if(currentMenu == BLOODMENU_LOBBY)
	{
		Menu_Lobby();
	}
	else if(currentMenu == BLOODMENU_MJOIN)
	{
		Menu_JoinGame();
	}
	else if(currentMenu == BLOODMENU_UMAPS)
	{
#if 0
		Menu_UserMaps();
#endif
	}
	else if(currentMenu == BLOODMENU_LOADING)
	{
		guimanager->DrawTextNoFlash("LOADING",125,80);
		guimanager->DrawTextNoFlash(thisGame->thisBoard,125,100);
		fontNum = 4257;
		lastMenu = BLOODMENU_LOADING; // cant escape out of the loadign screen
	}

	ThisMenuMain();
	DrawBloodDrips();
	FlushMenu();
	//DrawIcon();
}