//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
//-------------------------------------------------------------------------

// game_sector.cpp

#include "game.h"

/*
=================================
void Game::OperateSectors(void)
=================================
*/

void Game::OperateSectors(void)
{
	short neartagsector        = 0;
	short neartagwall          = 0;
	short neartagsprite        = 0;
	long neartaghitdist        = 0;
	bloodPlayer_t *thisPlayer  = gamePlayers.at(0);
	door_t           *thisDoor;
	doorSlideEvent_t *thisDoorSlide,*thisDoorSlideSpawn;
	xsectortype      *xsector1, *xsector2;
	xspritetype      *marker1, *marker2;
	build_func->neartag(thisPlayer->posx,thisPlayer->posy,thisPlayer->posz,thisPlayer->sectorNum,thisPlayer->ang,&neartagsector,&neartagwall,&neartagsprite,&neartaghitdist,1024L,3);
	
	for(int i = 0; i < worldEvents.size(); i++)
	{
		worldEvent_t *thisEvent = worldEvents.at(i);

		if(thisEvent->flagged != 0) // event is operating independenlly
			continue;

		switch(thisEvent->type)
		{
			case LEVEL_ZDOOR:
					thisDoor = (door_t *)thisEvent->data;
					if(neartagsector != thisDoor->sectNum)
						continue;
					if(thisEvent->flagged == 0)
						thisEvent->flagged = 1;
			break;

			case LEVEL_SLIDEDOOR:
				    thisDoorSlide = (doorSlideEvent_t *)thisEvent->data;
					xsector1      = build_func->GetXSector(thisDoorSlide->sectornum);
					marker1       = build_func->GetXSprite(xsector1->marker1);
					if(neartagsector == thisDoorSlide->sectornum)
					{
						thisEvent->flagged =1;
						// find any doors that should open at the same time.
						for(int j = 0; j < worldEvents.size(); j++)
						{
							worldEvent_t *spawnedEvent = worldEvents.at(j);

							if(spawnedEvent->type == LEVEL_SLIDEDOOR)
							{
								thisDoorSlideSpawn = (doorSlideEvent_t *)spawnedEvent->data;
								xsector2           = build_func->GetXSector(thisDoorSlideSpawn->sectornum);
								marker2            = build_func->GetXSprite(xsector2->marker1);

							//	if(xsector1->rxid == xsector2->rxid)
							//		spawnedEvent->flagged = 1;
							}
						}
					}
			break;
		}
	}
}


/*
=================================
void Game::SetSpriteStates(void)

Sets level to inital state
=================================
*/

void Game::SetSpriteStates(void)
{
	spritetype       *thisSprite,*sndSprite,*marker1,*marker2;
	sectortype       *thisSector;
	xsectortype      *xsector,*xsector1;
	xspritetype      *xsndsprite,*xsprite;
	worldEvent_t     *thisEvent;
	walltype         *thisWall;
	doorSlideEvent_t *slidedoorEvent;
	bloodPlayer_t    *thisPlayer = gamePlayers.at(0);
	int          sectNum = 0;
	int          spriteNum = 0;
	door_t       *thisDoor;
	char         temptxt[100];
	int wallNum   = 0;
	int doorwalls = 0;
	int wallnum   = 0;

	while( 1 )// Loop through all the game sectors
	{
		thisSprite    = build_func->Game_GetSprite(spriteNum);
		thisSector    = build_func->GetSector(sectNum);
		thisWall      = build_func->get_Wall(wallnum);
		xsector       = build_func->GetXSector(sectNum);
		xsector1      = build_func->GetXSector(sectNum-1);
		xsprite       = build_func->GetXSprite(spriteNum);
		marker1       = build_func->Game_GetSprite(xsector->marker1);
		marker2       = build_func->Game_GetSprite(xsector->marker2);
		thisEvent     = new worldEvent_t;
		try
		{
			if(thisWall != NULL)
			{
				if(thisWall->pal == 6 || thisWall->pal == 8)
					thisWall->pal = 0;
				wallnum++;
				continue;
			}
		}
		catch(...)
		{

		}

		if(thisSector == NULL && thisSprite == NULL)
		{
			numSectors = sectNum;
			return;
		}

		if(thisSprite != NULL)
		{
			if(thisSprite->pal == 6 || thisSprite->pal == 8)
				thisSprite->pal = 0;
			if(thisSprite->type == 1 && xsprite->data1 == 0)
			{
				thisPlayer->posx      = thisSprite->x;
				thisPlayer->posy      = thisSprite->y;
				thisPlayer->posz      = thisSprite->z;
				thisPlayer->ang       = thisSprite->ang;
				thisPlayer->sectorNum = thisSprite->sectnum;
			}

			if(thisSprite != NULL)
			{
				switch(thisSprite->picnum)
				{
					case 2825:
						thisSprite->flags = 100;
					break;
				}
				spriteNum++;
			}
		}

		if(thisSector != NULL)
		{
			if(thisSector->ceilingpal == 6 || thisSector->ceilingpal == 8)
				thisSector->ceilingpal = 0;
			if(thisSector->floorpal == 6 || thisSector->floorpal == 8)
				thisSector->floorpal = 0;
			switch(thisSector->lotag)
			{
				case LEVEL_DOORROTATE:
					if(marker1->x == 0 && marker1->y == 0 && marker1->z == 0)
						exit(0);
					// Move the door to a shut state
					MoveSector(thisSector,marker1->x,marker1->y,marker1->z,0,
										  marker1->x,marker1->y,marker1->z,marker1->ang,
									      xsector->rxid);
				break;
				case LEVEL_SLIDEDOOR:

					if(marker2 == NULL || marker1 == NULL)
						continue;
					slidedoorEvent = new doorSlideEvent_t(marker2->x,marker2->y,marker2->z,
						                                  marker1->x,marker1->y,marker1->z,
														  sectNum);
					
					// See if we need to play a sound effect
					for(int i = 0; i < numofsprites; i++)
					{
						sndSprite = build_func->Game_GetSprite(i);

						// found a SFX sprite
						if(sndSprite->sectnum == sectNum && sndSprite->type == 709 && i != 0)
						{
							xsndsprite = build_func->GetXSprite(i);
							slidedoorEvent->beginopen = xsndsprite->data1;
							slidedoorEvent->endopen   = xsndsprite->data2;

						//	sprintf(temptxt,"sndSprite sectnum = %d currsect = %d xsndsprite->data1 %d sprnum = %d\n", sndSprite->sectnum,sectNum,xsndsprite->data1,i);
						//	WriteLog(temptxt);
							break;
						}
					}

					// Move the door to a shut state
					MoveSector(thisSector,marker2->x,marker2->y,marker2->z,0,
										  marker1->x,marker1->y,marker1->z,0,
									      xsector->rxid);

					thisEvent->type      = LEVEL_SLIDEDOOR;
					thisEvent->data		 = malloc(sizeof(doorSlideEvent_t));
					thisEvent->data		 = slidedoorEvent;
					thisEvent->flagged	 = 0;
					thisEvent->switch_t  = 0;
					worldEvents.push_back(thisEvent);
					break;
				case LEVEL_ZDOOR:
					long minx=0,miny=0;
					long maxx=0,maxy=0;
					
					long startz = thisSector->ceilingz;
					long endz   = thisSector->floorz;

					thisDoor    = new door_t(minx,miny,maxx,maxy,startz,endz,sectNum);

					thisSector->ceilingz = thisSector->floorz; // set door state to closed
					thisEvent->type      = LEVEL_ZDOOR;
					thisEvent->data		 = malloc(sizeof(door_t));
					thisEvent->data		 = thisDoor;
					if(xsector->rxid  == 0)
						thisEvent->flagged	 = 0;
					else
						thisEvent->flagged	 = 12; // locked
					thisEvent->switch_t  = 0;
					worldEvents.push_back(thisEvent);
				break;
			}
			sectNum++;
		}
	}; 
}

/*
==================================================================================
void Game::RotatePoint(long *x, long *y, long angVect2, long startx, long starty, long pivx, long pivy)
==================================================================================
*/

void Game::RotatePoint(long *x, long *y, long angVect2, long startx, long starty, long pivx, long pivy)
{
	build_func->rotatepoint(pivx,pivy,startx,starty,angVect2,x,y);
}

/*
==============================================================================
void Game::MoveSector(sectortype *thisSector, long startx, long starty, long startz, long newx, long newy, long newz)
==============================================================================
*/

void Game::MoveSector(sectortype *thisSector, long startx, long starty, long startz, long oldang,
					  long newx, long newy, long newz, long newang, short rxid)
{
	int   i1                = 0x10000;
	if(newang != 0)
		i1                  = -0x10000;
	xsectortype *xsector    = build_func->GetXSector(thisSector->extra);
	short nWall             = thisSector->wallptr;
	int vectX               = newx - startx;
	int vectY               = newy - starty;
	int vectA               = oldang - newang;
	int vect1x              = mulscale16(i1, vectX);
	int vect1y              = mulscale16(i1, vectY);
	int angVect1            = mulscale16(i1, vectA);
	int angVect2            = mulscale16(i1, vectA);
	long pivx, pivy;
	xwalltype   *xwall;
	walltype    *thiswall;
	int startwallnum;
	int endwall;
	long x,y,x1,y1;
	long pivwall = nWall;

	if ( angVect2 )
	{
		for(int i = 0; i < thisSector->wallnum; i++, nWall++)
		{
			x = build_func->get_Wall(nWall)->x;
			y = build_func->get_Wall(nWall)->y;

			startwallnum = thisSector->wallptr;
			endwall      = startwallnum+thisSector->wallnum;
			xwall    = build_func->GetXWall(nWall);
			thiswall = build_func->get_Wall(nWall);

			x1 = build_func->get_Wall(thiswall->point2)->x;
			y1 = build_func->get_Wall(thiswall->point2)->y;

			if ( angVect2 )
				RotatePoint(&x, &y, angVect2, startx, starty,x,y);

			build_func->dragpoint(nWall, x + vect1x, y + vect1y);
		}
		return;
	}

	for(int i = 0; i < thisSector->wallnum; i++, nWall++)
	{
		x = build_func->get_Wall(nWall)->x;
		y = build_func->get_Wall(nWall)->y;

		xwall    = build_func->GetXWall(nWall);
		thiswall = build_func->get_Wall(nWall);

		if(thiswall->cstat & 0xC000)
		{
			if ( thiswall->cstat & 0x4000 ) // forward
			{
				build_func->dragpoint(nWall, x + vect1x, y + vect1y);
			}
			else if ( thiswall->cstat & 0x8000 ) // reverse
			{
				build_func->dragpoint(nWall, x - vect1x, y - vect1y);
			}
		}
	}
}

/*
=================================
void Game::RunOperators(void)
=================================
*/

void Game::RunOperators(void)
{
	short neartagsector        = 0;
	short neartagwall          = 0;
	short neartagsprite        = 0;
	long neartaghitdist        = 0;
	bloodPlayer_t    *thisPlayer  = gamePlayers.at(0);
	sectortype       *thisSector;
	doorSlideEvent_t *thisDoorSlide;
	door_t           *thisDoor;
	float goalx, gloaly;

	build_func->neartag(thisPlayer->posx,thisPlayer->posy,thisPlayer->posz,thisPlayer->sectorNum,thisPlayer->ang,&neartagsector,&neartagwall,&neartagsprite,&neartaghitdist,1024L,3);
	for(int i = 0; i < worldEvents.size(); i++)
	{
		worldEvent_t *thisEvent = worldEvents.at(i);

		switch(thisEvent->type)
		{
			case LEVEL_SLIDEDOOR:
				    thisDoorSlide = (doorSlideEvent_t *)thisEvent->data;

					if(thisEvent->flagged == 1)
					{
						// found out how many tics to have the door open
						goalx = abs((thisDoorSlide->closex - thisDoorSlide->openx)) / 20;

						thisDoorSlide->starttic = build_func->ftotalclock();
						thisDoorSlide->endtic   = build_func->ftotalclock() + goalx;

						thisSector = build_func->GetSector(thisDoorSlide->sectornum);
						thisEvent->flagged = 2;
					}
					else if(thisEvent->flagged == 2)
					{
						if(thisDoorSlide->endtic < thisDoorSlide->starttic)
						{
							PlaySfxSprite(thisDoorSlide->endopen);
							thisEvent->flagged = 3;
							continue;
						}

						if(thisDoorSlide->beginopen != 0)
						{
							PlaySfxSprite(thisDoorSlide->beginopen);
							thisDoorSlide->beginopen = 0;
						}

						thisSector = build_func->GetSector(thisDoorSlide->sectornum);

						MoveSector(thisSector,thisDoorSlide->openx + 20,thisDoorSlide->openy,thisDoorSlide->openz,0,
								   thisDoorSlide->openx,thisDoorSlide->openy,thisDoorSlide->openz,0,
					 			-1);

						thisDoorSlide->starttic += 1;
					}
			break;
			case LEVEL_ZDOOR:
			    thisDoor = (door_t *)thisEvent->data;
				thisSector = build_func->GetSector(thisDoor->sectNum);
				if(thisEvent->flagged == 1)
				{
					switch(thisEvent->switch_t)
					{
						case 0: // door state is now closed
							if(thisSector->ceilingz > thisDoor->startZ)
							{
								if(thisSector->ceilingz > thisDoor->endZ + 420)
								{
									//thread_t  *closeDoorThread;
									program_t *doorProg  = new program_t;

									thisSector->ceilingz = thisDoor->endZ;
									thisEvent->flagged   = 0;
									thisEvent->switch_t  = 1;

									printf("Flagging door\n");
									//doorProg->progBuffer = thread_CloseDoor(1000, i, thisDoor->startZ, thisDoor->endZ,
									//							   1000,2000);
									//doorProg->progSize   = strlen((char *)doorProg->progBuffer.c_str());

//									closeDoorThread = new thread_t(doorProg);

//									closeDoorThread->RunProgram();
								}
								else
								{
									thisSector->ceilingz-=420;
								}
							}
						break;

						case 1: // door state is now open
							if(thisSector->ceilingz < thisDoor->endZ)
							{
								if(thisSector->ceilingz < thisDoor->startZ - 420)
								{
									printf("Flagging door\n");
									thisSector->ceilingz = thisDoor->startZ;
									thisEvent->flagged   = 0;
									thisEvent->switch_t  = 0;
								}
								else
								{
									thisSector->ceilingz+=420;
								}
							}
						break;
					}
				}
				else if(thisEvent->flagged == 12)
				{
					thisSector->ceilingz = thisDoor->endZ;
				}
			break;
		}
	}
}


/*
======================================================
void Game::FOFDraw(int spnum, long smoothratio)

Special Thanks to Jonof for his help getting
me start with this.
======================================================
*/

void Game::FOFDraw(long x,long y,long z,int spnum, long smoothratio)
{
	bloodPlayer_t *thisPlayer = gamePlayers.at(0);
	spritetype  *effector1, *effector2,*thisSprite;
	sectortype  *mesector,  *friendsector, *playersector;
	xspritetype *effector1x, *effector2x;
	long fofx,fofy, fofz,sprnum;
	int fofmode;
	char temptxt[100];
	long distx, disty,distx1,disty1, oldz;
	long sostype=-1;
	boolean hack01 = FALSE;

	effector1     = build_func->Game_GetSprite(spnum); 
	effector1x    = build_func->GetXSprite(spnum);
	mesector      = build_func->GetSector(effector1->sectnum);
	playersector  = build_func->GetSector(thisPlayer->sectorNum);
	friendsector  = NULL;

	if(effector1->type == 9)  return; //fofmode = SOSEFFECT_UP;
	if(effector1->type == 10) return; //fofmode = SOSEFFECT_DOWN;	

	switch(effector1->picnum)
	{
		case SOSUP:
			// First step remove the cieling
			mesector->ceilingpicnum = 4054;

			// Find the friend sector
			for(int i = 0; i < this->numofsprites; i++)
			{
				thisSprite = build_func->Game_GetSprite(i);
				if(thisSprite == NULL)
					break;
				effector2x = build_func->GetXSprite(i);
				//sprintf(temptxt,"SOSUP: effector2x.data = %d picnum = %d type = %d, sprnum=%d\n",effector2x->data1,thisSprite->picnum,thisSprite->type, i);
				//WriteLog(temptxt);
				if((thisSprite->picnum == SOSDOWN) &&  (effector2x->data1 == effector1x->data1))
				{
					//sprintf(temptxt, "Found SOSDOWN! Match %d with %d\n", spnum,i);
					//WriteLog(temptxt);
					effector2               = thisSprite;
					friendsector	        = build_func->GetSector(thisSprite->sectnum);
					fofmode                 = 0;	
					sprnum                  = i;
					break;
				}
			}
		break;

		case SOSDOWN:
			// First Step remove the floor
			mesector->floorpicnum   = 4054;

			// Find the friend sector
			for(int i = 0; i <= this->numofsprites; i++)
			{
				thisSprite = build_func->Game_GetSprite(i);
				if(thisSprite == NULL)
					break;
				effector2x = build_func->GetXSprite(i);
				//sprintf(temptxt,"SOSDOWN: effector2x.data = %d picnum = %d type = %d, sprnum=%d\n",effector2x->data1,thisSprite->picnum, thisSprite->type, i);
				//WriteLog(temptxt);
				if((thisSprite->picnum == SOSUP) && (effector2x->data1 == effector1x->data1))
				{
					//sprintf(temptxt, "Found SOSUP! Match %d with %d\n", spnum,i);
					//WriteLog(temptxt);
					effector2			= thisSprite;
					friendsector		= build_func->GetSector(thisSprite->sectnum);
					fofmode             = 1;
					sprnum              = i;
					break;
				}
			}
		break;
	}

	if(friendsector == NULL || effector1 == effector2) // force a exit
	{
		sprintf(temptxt,"Failed to find matching effector for %d picnum = %d\n",effector1x->data1,effector1->picnum);
		FatalCrash(temptxt);
	}

	// See if sector is DIRECTLLY above, or off in no where
	if(FindDistance2D(effector2->x-effector1->x, effector2->y-effector1->y) < 3000)
		sostype = 1;
	else
		sostype = 2;

	if(sostype == 1)
	{
		/*
		distx = thisPlayer->posx - effector1->x;
		disty = thisPlayer->posy - effector1->y;
		*/
		distx = effector2->x + (thisPlayer->posx - effector1->x);
		disty = effector2->y + (thisPlayer->posy - effector1->y);
		if(!common->spriteSeePlayer(spnum))
			return;
		
		if(fofmode == 0)
		{
			oldz = mesector->ceilingz;
			mesector->ceilingz = friendsector->ceilingz;
			fofz  = (oldz - friendsector->ceilingz);
	//		fofz = effector2->z + (thisPlayer->posz - friendsector->ceilingz);
		}
		else
		{
			oldz = mesector->floorz;
			mesector->floorz = friendsector->floorz;
			fofz  = (oldz - friendsector->floorz);
	//		fofz = effector2->z + (thisPlayer->posz - friendsector->floorz);
		}

		//build_func->updatesector(effector2->x+distx,effector2->y+disty,&effector2->sectnum);
		build_func->displayLevel(distx,disty,z-(fofz/3),thisPlayer->ang,effector2->sectnum,65536);

		if(fofmode == 0)
			mesector->ceilingz = oldz;
		else
			mesector->floorz   = oldz;
	}
	else
	{
		distx = thisPlayer->posx - effector1->x;
		disty = thisPlayer->posy - effector1->y;

		if(!common->spriteSeePlayer(spnum))
			return;
	
		if(fofmode == 0)
		{
			oldz = mesector->ceilingz;
			mesector->ceilingz = friendsector->ceilingz;
			fofz = effector2->z + (thisPlayer->posz - friendsector->ceilingz);
		}
		else
		{
			oldz = mesector->floorz;
			mesector->floorz = friendsector->floorz;
			fofz = effector2->z + (thisPlayer->posz - friendsector->floorz);
		}

		build_func->displayLevel(effector2->x+distx,effector2->y+disty,fofz,thisPlayer->ang,thisPlayer->sectorNum,65536);

		if(fofmode == 0)
			mesector->ceilingz = oldz;
		else
			mesector->floorz   = oldz;
	}
}


/*
============================
void Game::SectorFOF(long x,long y,long z,long a,long h, long smoothratio)
============================
*/

void Game::SectorFOF(long x,long y,long z,long a,long h, long smoothratio)
{
	spritetype *thisSprite;
	int        spriteNum = 0;

	PushSectors();
	while( 1 )// Loop through all the game sprites
	{
		thisSprite = build_func->Game_GetSprite(spriteNum);

		if(thisSprite == NULL) // reached end of max sprites in level
			break;

		switch(thisSprite->picnum)
		{
			case SOSUP:
				FOFDraw(x,y,z,spriteNum,smoothratio);
			break;
			case SOSDOWN:
				FOFDraw(x,y,z,spriteNum,smoothratio);
			break;
		}
		spriteNum++;
	};
	PopSectors();
}
