//-------------------------------------------------------------------------
/*
Copyright (C) 2005 - Justin Marshall

This file is part of the blood reconstruction project.

WinBlood is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  

See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

PORTIONS OF CODE FROM DUKE NUKEM 3D SOURCE CODE
*/
//-------------------------------------------------------------------------

// game_shared.cpp

#include "game.h"

int totalclock;

Game            *thisGame;
guiManager      *guimanager;
build_func_t    *build_func;
fileManager_t   *fileManager;
gameMenu_t      *menuManager;
soundManager_t  *soundManager;
kenNet_t        *netManager;
console_t       *console;
sectorManager_t *sectorManager;
jCommon         *common;

#define GAME_BUILD "Game build: " __DATE__ " " __TIME__ 
// RunFrame
// Calls gamemain

int RunFrame(int totalclock)
{
	return thisGame->RunFrame(totalclock);
}

int GameShutdown()
{
	thisGame->Shutdown();
	return -1;
}

// Game_API
// Function that interacts with the build system

Game_Exports *Game_API(Game_Imports *gi)
{
	Game_Exports *ge = new Game_Exports;
	build_func       = new build_func_t;

	printf(GAME_BUILD);
	printf("\n");
	ge->RunFrame     = RunFrame;
	ge->GameShutdown = GameShutdown; 
	thisGame         = new Game();

	build_func->nextpage		= gi->nextpage;
	build_func->build_KeyDown	= gi->build_Keydown;
	build_func->Load_Pallete    = gi->Load_Pallete;
	build_func->rotatesprite    = gi->rotatesprite;
	build_func->clearview       = gi->clearview;
	build_func->loadboard       = gi->loadboard;
	build_func->SDL_Quit		= gi->SDL_Quit;
	build_func->Decrypt_Map     = gi->Decrypt_Map;
	build_func->displayLevel    = gi->displayLevel;
	build_func->P_Movecode      = gi->P_Movecode;
	build_func->clipmove        = gi->clipmove;
	build_func->mulscale		= gi->mulscale;
	build_func->pushmove		= gi->pushmove;
	build_func->ftotalclock     = gi->ftotalclock;
	build_func->Set_Brightness  = gi->Set_Brightness;
	build_func->getzrange		= gi->getzrange;
	build_func->Game_GetSprite  = gi->Game_GetSprite;
	build_func->Justin_SetSky   = gi->Justin_SetSky;
	build_func->build_ClearKey  = gi->build_ClearKey;
	build_func->GetSector       = gi->GetSector;
	build_func->neartag			= gi->neartag;
	build_func->hitscan		    = gi->hitscan;
	build_func->krand			= gi->krand;
	build_func->setviewback	    = gi->setviewback;
	build_func->setviewtotile   = gi->setviewtotile;
	build_func->getTileX	    = gi->getTileX;
	build_func->getTileY	    = gi->getTileY;
	build_func->loadtile		= gi->loadtile;
	build_func->getSinTable		= gi->getSinTable;
	build_func->getParam		= gi->getParam;
	build_func->get_Wall        = gi->get_Wall;
	build_func->cansee			= gi->cansee;
	build_func->drawline256		= gi->drawline256;
	build_func->GetXSprite		= gi->GetXSprite;
	build_func->getincangle     = gi->getincangle;
	build_func->GetXSector      = gi->GetXSector;
	build_func->GetXWall        = gi->GetXWall;
	build_func->dragpoint       = gi->dragpoint;
	build_func->rotatepoint     = gi->rotatepoint;
	build_func->InsertSprite    = gi->InsertSprite;
	build_func->spawnsprite     = gi->spawnsprite;
	build_func->deletesprite    = gi->deletesprite;
	build_func->unsafeGetSprite = gi->unsafeGetSprite;
	build_func->updatesector    = gi->updatesector;
	build_func->build_gotpic    = gi->build_gotpic;
	build_func->inside          = gi->inside;
	build_func->getangle        = gi->getangle;
	build_func->build_resetgotpic   = gi->build_resetgotpic;
	build_func->playerTouchSprite   = gi->playerTouchSprite;
	build_func->changespritesect    = gi->changespritesect;
	build_func->Change_ScreenInfo   = gi->Change_ScreenInfo;
	build_func->KBClearLastScanCode = gi->KBClearLastScanCode;
	memcpy(build_func->sintable,gi->sintable,2048);

	// Create our managers here...
	guimanager    = new guiManager;
    fileManager   = new fileManager_t();
	menuManager   = new gameMenu_t();
	soundManager  = new soundManager_t();
	netManager    = new kenNet_t();
	console       = new console_t();
	sectorManager = new sectorManager_t();
	common        = new jCommon;
	return ge;
}